#pragma rtGlobals=1		// Use modern global access method.
 
 //	3D_path

//	Author: Tomoko Narazaki, Kozue Shiomi
//	e-mail: naratomoz@gmail.com, shiomikozue@gmail.com

//	----------------------------------------------------------------------------------------------
//	All code was written by Tomoko Narazaki & Kozue Shiomi. The author takes no responsibility for its use or misuse. Copies of this 
//	code should be distributed with their original 3D_path identification headers. Any code derived from 3D_path 
//	should not be sold for commercial purposes. Please cite the use of 3D_path whenever possible.
//	----------------------------------------------------------------------------------------------

//	Reference 
//	----------------------------------------------------------------------------------------------
//	Shiomi K., Narazaki T., Sato K., Shimatani K., Arai N., Ponganis P.J., Miyazaki N. (2010) Data-processing artefacts in three-dimensional 
//			dive path reconstruction from geomagnetic and acceleration data. Aquatic Biology, 8, 299-304
//   Narazaki T & Shiomi K. (2010) Reconstruction of 3-D path (ThreeD_path).  
//		       http://japan-biologgingsci.org/home/macro/threed_path/
//	----------------------------------------------------------------------------------------------

//	History
//
//	Version 1.0 April 09, 2010
//		*** Public release ***
//
//   Version 1.01 July 30, 2012
//		*** minor error (not influeincing the results) corrected  ***
//         Print " lateral magnetismF"+nameofwave($surgeM)
//         ˁ@Print " longitudinal magnetismF"+nameofwave($surgeM)
//
// **************************************************
//  *** Please do NOT use Version 1.0 & 1.01. They include crucial errors.***
// **************************************************

//   Version 2.0 September 09, 2019
//		*** equations for ROLL calculation corrected  ***
//		*** attachment angles for ROLL and HEADING added  as parameters ***
//		*** authors' e-mail addresses changed  ***
//		*** URL  changed  ***

 ///////////////////////Correction of acceleration data///////////////////////
 Function GCorrect(gdata)
      wave gdata
  
      variable i=0
      
      do
         if (gdata[i]>1)
            gdata[i]=1
         elseif (gdata[i]< (-1))
            gdata[i]=-1
         endif
       i+=1
     while(i<numpnts(gdata))
 End
//////////////////////////////////////////////////////////////////
 
///////////////////////Sampling interval/////////////// 
Function Sampling(surgeG, swayG,heaveG, surgeM, swayM, heaveM, Sp, De)
wave surgeG, swayG,heaveG, surgeM, swayM, heaveM, Sp, De

Make/N=8 Sample
      Sample[0]=deltax(surgeG)
      Sample[1]=deltax(swayG)
      Sample[2]=deltax(heaveG)
      Sample[3]=deltax(surgeM)
      Sample[4]=deltax(swayM)
      Sample[5]=deltax(heaveM)
      Sample[6]=deltax(Sp)
      Sample[7]=deltax(De)
      WaveStats/Q Sample
 
 if (V_maxloc == 0)
 	Duplicate/O surgeG sampleint
 elseif (V_maxloc==1)
 	Duplicate/O swayG sampleint
  elseif (V_maxloc==2)
 	Duplicate/O heaveG sampleint
  elseif (V_maxloc==3)
 	Duplicate/O surgeM sampleint
  elseif (V_maxloc==4)
 	Duplicate/O swayM sampleint
  elseif (V_maxloc==5)
 	Duplicate/O heaveM sampleint
  elseif (V_maxloc==6)
 	Duplicate/O Sp sampleint
  elseif (V_maxloc==7)
 	Duplicate/O De sampleint
 endif
 
 sampleint=0
 
 End
 //////////////////////////////////////////////////////////////////

      
/////////////////////// Heading calculation////////////////////////////
Function calHead(surgeG, swayG,heaveG, surgeM, swayM, heaveM, sampleint,matPo, matRo, matHo)

      wave surgeG,swayG,heaveG,surgeM,swayM,heaveM,sampleint,matPo, matRo, matHo
      
      variable i
      
      NVAR accel
      Duplicate/O sampleint surgeGx, swayGx, heaveGx, surgeMx, swayMx, heaveMx, Pitch, Roll, Head
     
      surgeGx=surgeG(x)/accel
      swayGx=swayG(x)/accel
      heaveGx=heaveG(x)/accel
      
      surgeMx=surgeM(x)
      swayMx=swayM(x)
      heaveMx=heaveM(x)
      
      GCorrect(surgeGx)
      GCorrect(swayGx)
      GCorrect(heaveGx)
      
      Pitch=NaN; Roll=NaN; Head=NaN

      i=0
      do     
            make matM={surgeMx[i],swayMx[i],heaveMx[i]}
            make matG={surgeGx[i],swayGx[i],heaveGx[i]}
            
            MatrixOP matAw=Inv(matHo) x Inv(matPo) x Inv(matRo) x matG
            
             if (matAw[0]>1)
                matAw[0]=1
             elseif (matAw[0]<-1)
                matAw[0]=-1
             endif  
            
            Pitch[i]=-asin(matAw[0])
	      
            if((matAw[1]<0)&&(matAw[2]==0))
		   Roll[i] = pi/2

	      elseif((matAw[1]>0)&&(matAw[2]==0))
		   Roll[i] = -pi/2

	      elseif(matAw[1]<=0)
		   Roll[i] = atan2(matAw[1],matAw[2])+pi

	      elseif(matAw[1]>0)
		   Roll[i] = atan2(matAw[1],matAw[2])-pi

	      endif
	      
            make matP={{cos(Pitch[i]),0,-sin(Pitch[i])}, {0,1,0}, {sin(Pitch[i]), 0, cos(Pitch[i])}}
            make matR={{1,0,0},{0,cos(Roll[i]),-sin(Roll[i])},{0,sin(Roll[i]),cos(Roll[i])}}
            
            MatrixOP matHB=Inv(matP) x Inv(matR) x Inv(matHo) x Inv(matPo) x Inv(matRo) x matM
            
             Head[i]=atan2(matHB[1],matHB[0])
     	       
            i+=1
            KillWaves matR, matP, matM, matG, matAw, matHB
            
      while(i<numpnts(Head))
      KillWaves matPo, matRo, matHo
      
 End
 //////////////////////////////////////////////////////////////////

  
 //////////////////////Brank correction///////////////////////
 Function BrankCorrect(del)
      wave del
  
      variable val,val1,i=1
      del[0]=0
      do
         val=del[i]
         val1=del[i+1]
         if (numtype(val)==0)
            del[i]=del[i]
         else
           if (numtype(val1)==0)
              del[i]=(del[i-1]+del[i+1])/2
           else
              del[i]=del[i-1]
           endif
         endif
         i+=1
     while(i<numpnts(del))
 End
 //////////////////////////////////////////////////////////////////

 
 ///////////////////path reconstruction //////////////////////
 Function ThreeD(Pitch,Head,Sp,De)
   wave Pitch,Head,Sp,De
   variable i=0
   Duplicate/O sampleint delX, delY, X3, Y3, Z3, Spx,Dex
   
   Spx=Sp(x)
   Dex=De(x)
            
   do
  		 if (Head [i] < -pi)
       	    Head[i] += (2*pi)
     		 elseif  (Head [i] > pi)
      		     Head [i] -=(2*pi)
     	       endif
     	       i += 1
    while (i < numpnts(Head))
    
   delX[0]=0
   delX[1,]=Spx[p]*cos(Pitch[p-1])*sin(Head[p-1])
   delY[0]=0
   delY[1,]=Spx[p]*cos(Pitch[p-1])*cos(Head[p-1])
   
    
    BrankCorrect(delX)
    BrankCorrect(delY)
    
    
    X3[0]=0
    Y3[0]=0  
    X3[1,]=X3[p-1]+delX[p]
    Y3[1,]=Y3[p-1]+delY[p]
    Z3=-Dex
    
  End    
  //////////////////////////////////////////////////////////////////

  
  /////////////////Macro///////////////////////////////
  Macro ThreeD_path()
    Print ""
    Print "********** start '3D_path'  **********"
    Print ""
        
    variable/G accel
    NewPanel/N=acceleration_type/W=(400,100, 650, 230)
    CheckBox type1 mode=1, title="}   1 G ", proc=Check, pos={60,50}
    CheckBox type2 mode=1, title="} 9.8 m/s\S2  \M", proc=Check, pos={60,70}
    DrawText 16,32,"Select the unit of the acceleration data"
    Button btn1 pos={140, 100}, title="Continue", proc=BTN1, size={90,20}

  End  
  
  Proc Attachment(adjP,adjR,adjH,decli)
    variable adjP,adjR,adjH,decli
    Prompt adjP," attachment pitch angleijF"
    Prompt adjR," attachment roll angleijF"
    Prompt adjH," attachment heading angleijF"
    Prompt decli," geomagnetic declinationijF"
    
    adjP*=(pi/180)
    adjR*=(pi/180)
    adjH*=(pi/180)
    decli*=(pi/180)
    
   variable/G GadjP=adjP
   variable/G GadjR=adjR
   variable/G GadjH=adjH
   variable/G Gdecli=decli
    
    make matPo ={{cos(adjP),0,-sin(adjP)}, {0,1,0}, {sin(adjP), 0, cos(adjP)}}
    make matRo ={{1,0,0}, {0,cos(adjR),-sin(adjR)}, {0,sin(adjR), cos(adjR)}}
    make matHo ={{cos(adjH),sin(adjH),0},  {-sin(adjH), cos(adjH),0},{0,0,1}}
      
    Execute/Q "Path()"

  EndMacro
 //////////////////////////////////////////////////////////////////
   
//////////////////////////////////////////////////////////////////
  Proc Path(surgeG, surgeM, swayG, swayM, heaveG, heaveM, Sp, De)
    string surgeG, surgeM, swayG, swayM, heaveG, heaveM, Sp, De
    Prompt surgeG,  " longitudinal accelerationF",popup WaveList("*",";","")	
    Prompt surgeM, " longitudinal magnetismF",popup WaveList("*",";","")	
    Prompt swayG,   " lateral accelerationF",popup WaveList("*",";","")	
    Prompt swayM,  " lateral magnetismF",popup WaveList("*",";","")	
    Prompt heaveG, " dorsoventral accelerationF",popup WaveList("*",";","")	
    Prompt heaveM, " dorsoventral magnetismF",popup WaveList("*",";","")
    Prompt Sp, " speedF",popup WaveList("*",";","")
    Prompt De, " depthF",popup WaveList("*",";","")	
          
   PauseUpdate; Silent 1    
    Print "attachment pitch angleijF@"+ num2str(GadjP*180/pi)
    Print "attachment roll angleijF@"+ num2str(GadjR*180/pi)
    Print "attachment heading angleijF@"+ num2str(GadjH*180/pi)
    Print "geomagnetic declinationijF@" + num2str(Gdecli*180/pi)
    Print "---------------used waves -----------------"
    Print " longitudinal accelerationF"+nameofwave($surgeG)
    Print " lateral accelerationF"+nameofwave($swayG)
    Print " dorsoventral accelerationF"+nameofwave($heaveG)
    Print " longitudinal magnetismF"+nameofwave($surgeM)
    Print " lateral magnetismF"+nameofwave($swayM)
    Print " dorsoventral magnetismF"+nameofwave($heaveM)
    Print " speedF"+nameofwave($Sp)
    Print " depthF"+nameofwave($De)
    Print "----------------------------------------"
      
      Sampling($surgeG, $swayG, $heaveG, $surgeM, $swayM, $heaveM, $Sp, $De)
      
      calHead($surgeG, $swayG, $heaveG, $surgeM, $swayM, $heaveM, sampleint,matPo, matRo, matHo)
    
      Head+=Gdecli
      
      ThreeD(Pitch, Head, $Sp, $De)  
    
      Pitch*=(180/pi)  
      Roll*=(180/pi)
      Head*=(180/pi)
    
      Killwaves sampleint, surgeGx, swayGx, heaveGx, surgeMx, swayMx, heaveMx, Spx, Dex, Sample
      
      Print ""
      Print "********** End '3D_path'  **********"
      Print ""
      
  EndMacro
  //////////////////////////////////////////////////////////////////

  
  /////////////  acceleration data type ///////////////////////
  Function Check(ctrName,value)
     string ctrName
     variable value
     
     NVAR accel= root:accel 
     
     strswitch (ctrName)
         case "type1":
             accel=1
             break
         case "type2":
             accel=9.8
             break
     endswitch    
     
     CheckBox type1 value= accel==1
     CheckBox type2 value= accel==9.8
  
  End   
  //////////////////////////////////////////////////////////////////

  
 /////////////  attachment angles  ///////////////////////
 Function BTN1(ctrName)
    string ctrName
    
      KillWindow acceleration_type
      Execute/Q "Attachment()"

  End 